-- Code by SineMatic(YT)
-- If you want to use this code, let me know

-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}

local firstReset = false
local dataTemp = nil
local currentFilter = nil
local config = nil
local model = nil
local sleepingMode = false

local function updateFilter()
  obj:queueGameEngineLua([[
    if be:getPlayerVehicleID(0) == ]]..tostring(obj:getId())..[[ then
      core_sounds.cabinFilterStrength = ]]..tostring(currentFilter)..[[
    end
  ]])
end

local function getData()
  dataTemp = jsonReadFile("vehicles/"..model.."/dynamicFilter_"..config..".json")
  
  if not dataTemp then
    log("W", "dynamicCabinFilter", "Failed to find the dynamicFilter_"..config..".json file. Unloading the dynamicCabinFilter.lua module.") 
    -- print(config)
    extensions.unload("dynamicCabinFilter")
    return
  end
  
  for groupName, groupData in pairs(dataTemp) do
    for triggerName, triggerData in pairs(groupData["triggers"]) do
      if triggerData["role"] == "active" then
        triggerData["flag"] = 0
      end
    end
  end
  currentFilter = v.data.sounds.cabinFilterCoef
end

local function loadValues(configName)
  model = v.data.model
  currentFilter = v.data.sounds.cabinFilterCoef
  local startIndex1, endIndex1 = string.find(configName, "vehicles/"..model.."/")
  if startIndex1 == nil then
    config = configName
  else
    config = string.sub(configName, endIndex1 + 1, -4)
  end
  getData()
end

local function onExtensionLoaded()
  firstReset = true
  obj:queueGameEngineLua([[
    local id = ]]..tostring(obj:getId())..[[
    
    local vehicleData = core_vehicle_manager.getVehicleData(id)
    local foundConfig = vehicleData.config.partConfigFilename
    
    if foundConfig == "settings/default.pc" then
      foundConfig = nil 
      local model = vehicleData.config.model  
      local defaultConfigParts = jsonReadFile("settings/default.pc").parts 
      for key, value in pairs(core_vehicles.getModel(model).configs) do 
        if foundConfig then break end 
        foundConfig = key 
        local data = jsonReadFile("vehicles/"..model.."/"..key..".pc") 
        for slot, partName in pairs(data.parts) do 
          if not foundConfig then break end 
          if (defaultConfigParts[slot] ~= partName) then 
            foundConfig = nil 
            break  
          end 
        end 
      end 
    end
    
    be:getObjectByID(id):queueLuaCommand('extensions.dynamicCabinFilter.loadValues("'..foundConfig..'")')
  ]])
end

local function onReset()
  if not firstReset then
    getData()
    updateFilter()
  else
    firstReset = false
  end
end

local function updateGFX(dt)
  if sleepingMode then return end
  
  local updateValue = currentFilter
  
  for groupName, groupData in pairs(dataTemp) do
     for triggerName, triggerData in pairs(groupData["triggers"]) do
      if triggerData["role"] == "active" then
        if load(triggerData["function"])()() == 1 then 
          if triggerData["flag"] == 0 then
            triggerData["flag"] = 1
            updateValue = updateValue * (1 - groupData["fullDecrease"])
          end
          goto continue
        elseif triggerData["flag"] == 1 then
          triggerData["flag"] = 0
          updateValue = updateValue / (1 - groupData["fullDecrease"])
        end
      elseif load(triggerData["function"])()() == true then
        if not triggerData["partialDecrease"] then
          updateValue = updateValue * (1 - groupData["fullDecrease"])
          dataTemp[groupName] = nil
          goto continue
        else
          local d1 = groupData["fullDecrease"]
          local d2 = triggerData["partialDecrease"]
          updateValue = updateValue * (1 - d1*d2)
          groupData["fullDecrease"] = d1 * (d2 - 1) / (d1*d2 - 1)
          groupData["triggers"][triggerName] = nil
        end
      end
    end
    ::continue::
  end
  
  if updateValue ~= currentFilter then
    currentFilter = updateValue
    updateFilter()
  end
end

local function onPlayersChanged(switchedToThisVehicle)
  if switchedToThisVehicle then
    if currentFilter then
      sleepingMode = false
      updateFilter()
    end
  else
    sleepingMode = true
  end
end

M.onExtensionLoaded = onExtensionLoaded
M.onPlayersChanged = onPlayersChanged
M.updateFilter = updateFilter
M.loadValues = loadValues
M.updateGFX = updateGFX
M.onReset = onReset

return M